#include <Windows.h>
#include <windowsx.h>
#include <tchar.h>
#include <stdio.h>
#include <malloc.h>
#include <string.h>
#include <stdint.h>
#include <process.h>
#include "resource.h"
#include <setupapi.h>
#ifndef _NTSTATUS_DEFINED
#define _NTSTATUS_DEFINED
typedef LONG    NTSTATUS;
typedef LONG    *PNTSTATUS;
#endif
#include <hidsdi.h>
#pragma comment(lib, "hid.lib")
#pragma comment(lib, "setupapi.lib")

#define USB_VENDOR_ID          0x4C53
#define USB_DEVICE_ID          0x0010

#define REPORT_ID_RELAY0       0x40
#define RELAY0_REPORT_LENGTH   0x01

#define REPORT_ID_RELAY1       0x41
#define RELAY1_REPORT_LENGTH   0x01


HANDLE            hHID = INVALID_HANDLE_VALUE;
HWND              hDlg;

HANDLE OpenHID(int vendor, int product)
{
    GUID                                hidGuid;        /* GUID for HID driver */
    HDEVINFO                            deviceInfoList;
    SP_DEVICE_INTERFACE_DATA            deviceInfo;
    SP_DEVICE_INTERFACE_DETAIL_DATA     *deviceDetails = NULL;
    DWORD                               size;
    int                                 i, openFlag = 0;  /* may be FILE_FLAG_OVERLAPPED */
    HANDLE                              handle = INVALID_HANDLE_VALUE;
    HIDD_ATTRIBUTES                     deviceAttributes;
                
    HidD_GetHidGuid(&hidGuid);
    deviceInfoList = SetupDiGetClassDevs(&hidGuid, NULL, NULL, DIGCF_PRESENT | DIGCF_DEVICEINTERFACE);
    deviceInfo.cbSize = sizeof(deviceInfo);
    for (i=0; ;i++)
    {
        if (handle != INVALID_HANDLE_VALUE)
        {
            CloseHandle(handle);
            handle = INVALID_HANDLE_VALUE;
        }
        if (!SetupDiEnumDeviceInterfaces(deviceInfoList, 0, &hidGuid, i, &deviceInfo))
        {
            break;  /* no more entries */
        }
        /* first do a dummy call just to determine the actual size required */
        SetupDiGetDeviceInterfaceDetail(deviceInfoList, &deviceInfo, NULL, 0, &size, NULL);
        if (deviceDetails != NULL)
        {
            free(deviceDetails);
        }
        deviceDetails = malloc(size);
        if (NULL == deviceDetails)
        {
            return INVALID_HANDLE_VALUE;
        }
        memset(deviceDetails, 0, size);
        deviceDetails->cbSize = 5;
        /* this call is for real: */
        if (TRUE != SetupDiGetDeviceInterfaceDetail(deviceInfoList, &deviceInfo, deviceDetails, size, &size, NULL))
        {
           continue;
        }
        /* attempt opening for R/W -- we don't care about devices which can't be accessed */
        handle = CreateFile(deviceDetails->DevicePath, GENERIC_READ|GENERIC_WRITE, FILE_SHARE_READ|FILE_SHARE_WRITE, NULL, OPEN_EXISTING, openFlag, NULL);
        if (handle == INVALID_HANDLE_VALUE)
        {
            continue;
        }
        deviceAttributes.Size = sizeof(deviceAttributes);
        HidD_GetAttributes(handle, &deviceAttributes);
        if (deviceAttributes.VendorID != vendor || deviceAttributes.ProductID != product)
        {
            CloseHandle(handle);
            handle = INVALID_HANDLE_VALUE;
            continue;   /* ignore this device */
        }
        break;  /* we have found the device we are looking for! */
    }
    SetupDiDestroyDeviceInfoList(deviceInfoList);
    if (deviceDetails != NULL)
    {
        free(deviceDetails);
    }
    return handle;
}


void onRel0(HWND hDlg, BOOLEAN on)
{
    unsigned char report[RELAY0_REPORT_LENGTH + 1];

    if (INVALID_HANDLE_VALUE != hHID)
    {
        report[0] = REPORT_ID_RELAY0;
        report[1] = (unsigned char) on;
        if (HidD_SetFeature(hHID, report, RELAY0_REPORT_LENGTH + 1))
        {
            return;
        }
    }
    MessageBox(hDlg, TEXT("Relay not detected!"), TEXT("No Device"), MB_ICONWARNING | MB_OK);
    DestroyWindow(hDlg);
}

void onRel1(HWND hDlg, BOOLEAN on)
{
    unsigned char report[RELAY1_REPORT_LENGTH + 1];

    if (INVALID_HANDLE_VALUE != hHID)
    {
        report[0] = REPORT_ID_RELAY1;
        report[1] = (unsigned char) on;
        if (HidD_SetFeature(hHID, report, RELAY1_REPORT_LENGTH + 1))
        {
            return;
        }
    }
    MessageBox(hDlg, TEXT("Relay not detected!"), TEXT("No Device"), MB_ICONWARNING | MB_OK);
    DestroyWindow(hDlg);
}



INT_PTR CALLBACK DialogProc(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
    HWND                hList;
    long                temp;

    switch(uMsg)
    {
        case WM_COMMAND:
            switch(LOWORD(wParam))
            {
                case ID_ON0:   onRel0(hDlg, TRUE); 
                               return TRUE;
                case ID_OFF0:  onRel0(hDlg, FALSE); 
                               return TRUE;
                case ID_ON1:   onRel1(hDlg, TRUE); 
                               return TRUE;
                case ID_OFF1:  onRel1(hDlg, FALSE); 
                               return TRUE;
            }
            break;

        case WM_CLOSE:
            if (MessageBox(hDlg, TEXT("Close the program?"), TEXT("Close"), MB_ICONQUESTION | MB_YESNO) == IDYES)
            {
                DestroyWindow(hDlg);
                if (INVALID_HANDLE_VALUE != hHID)
                {
                    CloseHandle(hHID);
                    hHID = INVALID_HANDLE_VALUE;
                }
            }
            return TRUE;

        case WM_DESTROY:
            PostQuitMessage(0);
            return TRUE;
    }
    return FALSE;
}


static void initDialog(HWND hDlg)
{
    unsigned char report[RELAY0_REPORT_LENGTH + 1];

    hHID = OpenHID(USB_VENDOR_ID, USB_DEVICE_ID);
    if (INVALID_HANDLE_VALUE != hHID)
    {
        report[0] = REPORT_ID_RELAY0;
        if (HidD_GetFeature(hHID, report, RELAY0_REPORT_LENGTH + 1))
        {
            if (report[1])
            {
                Button_SetCheck(GetDlgItem(hDlg, ID_ON0), TRUE);
            }
            else
            {
                Button_SetCheck(GetDlgItem(hDlg, ID_OFF0), TRUE);
            }
        }
        report[0] = REPORT_ID_RELAY1;
        if (HidD_GetFeature(hHID, report, RELAY1_REPORT_LENGTH + 1))
        {
            if (report[1])
            {
                Button_SetCheck(GetDlgItem(hDlg, ID_ON1), TRUE);
            }
            else
            {
                Button_SetCheck(GetDlgItem(hDlg, ID_OFF1), TRUE);
            }
        }
    }
    else
    {
        MessageBox(hDlg, TEXT("Relay not detected!"), TEXT("No Device"), MB_ICONWARNING | MB_OK);
        DestroyWindow(hDlg);
    }
}


int WINAPI _tWinMain(HINSTANCE hInst, HINSTANCE h0, LPTSTR lpCmdLine, int nCmdShow)
{
    MSG msg;
    BOOL ret;

    hDlg = CreateDialogParam(hInst, MAKEINTRESOURCE(ID_DIALOG), 0, DialogProc, 0);
    ShowWindow(hDlg, nCmdShow);
    initDialog(hDlg);

    while ((ret = GetMessage(&msg, 0, 0, 0)) != 0) 
    {
        if (ret == -1) return -1;

        if (!IsDialogMessage(hDlg, &msg)) 
        {
            TranslateMessage(&msg);
            DispatchMessage(&msg);
        }
    }
    return 0;
}
